@@ -4,7 +4,7 @@ use std::{
     sync::Arc,
 };
 
-use crate::ui::{Drawer, SelectEntryDraw};
+use crate::mode::{fuzzy_matches, FilterEntry};
 
 pub mod git;
 pub mod hg;
@@ -58,9 +58,24 @@ pub struct RevisionInfo {
 
 #[derive(Clone)]
 pub struct RevisionEntry {
+    pub selected: bool,
     pub name: String,
     pub status: FileStatus,
 }
+impl RevisionEntry {
+    pub fn new(name: String, status: FileStatus) -> Self {
+        Self {
+            selected: false,
+            name,
+            status,
+        }
+    }
+}
+impl FilterEntry for RevisionEntry {
+    fn fuzzy_matches(&self, pattern: &str) -> bool {
+        fuzzy_matches(&self.name, pattern)
+    }
+}
 
 pub struct LogEntry {
     pub graph: String,
@@ -70,55 +85,32 @@ pub struct LogEntry {
     pub refs: String,
     pub message: String,
 }
+impl FilterEntry for LogEntry {
+    fn fuzzy_matches(&self, pattern: &str) -> bool {
+        fuzzy_matches(&self.message, pattern)
+            || fuzzy_matches(&self.refs, pattern)
+            || fuzzy_matches(&self.author, pattern)
+            || fuzzy_matches(&self.date, pattern)
+            || fuzzy_matches(&self.hash, pattern)
+    }
+}
 
 pub struct BranchEntry {
     pub name: String,
     pub checked_out: bool,
 }
+impl FilterEntry for BranchEntry {
+    fn fuzzy_matches(&self, pattern: &str) -> bool {
+        fuzzy_matches(&self.name, pattern)
+    }
+}
 
 pub struct TagEntry {
     pub name: String,
 }
-
-#[derive(Clone)]
-pub struct SelectableRevisionEntry {
-    pub selected: bool,
-    pub name: String,
-    pub status: FileStatus,
-}
-impl From<RevisionEntry> for SelectableRevisionEntry {
-    fn from(other: RevisionEntry) -> Self {
-        Self {
-            selected: false,
-            name: other.name,
-            status: other.status,
-        }
-    }
-}
-impl SelectEntryDraw for SelectableRevisionEntry {
-    fn draw(&self, drawer: &mut Drawer, _: bool, _: bool) -> usize {
-        const NAME_TOO_LONG_PREFIX: &str = "...";
-
-        let name_available_width = (drawer.viewport_size.0 as usize)
-            .saturating_sub(2 + 1 + FileStatus::max_len() + 1 + 1 + NAME_TOO_LONG_PREFIX.len());
-
-        let (name_prefix, trimmed_name) =
-            match self.name.char_indices().nth_back(name_available_width) {
-                Some((i, _)) => (NAME_TOO_LONG_PREFIX, &self.name[i..]),
-                None => ("", &self.name[..]),
-            };
-
-        let selected_text = if self.selected { '+' } else { ' ' };
-        drawer.fmt(format_args!(
-            "{} [{:>width$}] {}{}",
-            selected_text,
-            self.status.as_str(),
-            name_prefix,
-            trimmed_name,
-            width = FileStatus::max_len(),
-        ));
-
-        1
+impl FilterEntry for TagEntry {
+    fn fuzzy_matches(&self, pattern: &str) -> bool {
+        fuzzy_matches(&self.name, pattern)
     }
 }
 
